/*
 * libkysdk-system's Library
 *
 * Copyright (C) 2022, Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yunhe Liu <liuyunhe@kylinos.cn>
 *
 */

#ifndef KDK_SYSTEM_HW_CPU_H__
#define KDK_SYSTEM_HW_CPU_H__
/**
 * @file libkycpu.h
 * @author liuyunhe (liuyunhe@kylinos.cn)
 * @brief CPU信息
 * @version 0.1
 * @date 2021-11-17
 * 
 * @copyright Copyright (c) 2021
 * @defgroup libkysdk-hardware
 * @{
 * 
 */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 获取CPU架构
 * 
 * @return const char* 架构信息，如"x86_64"；返回的是const char*，不要free
 */
extern const char* kdk_cpu_get_arch();

/**
 * @brief 获取CPU制造厂商
 * 
 * @return const char* 制造厂商字符串，如“GenuineIntel”；返回的是const char*，不要free
 */
extern const char* kdk_cpu_get_vendor();

/**
 * @brief 获取CPU型号
 * 
 * @return const char* CPU型号名称，如“Intel(R) Core(TM) i7-4790K CPU @ 4.00GHz”；返回的是const char*，不要free
 */
extern const char* kdk_cpu_get_model();

/**
 * @brief 获取CPU额定主频
 * 
 * @return const char* 额定主频赫兹数，如“1794.742”，单位是MHz；返回的是const char*，不要free
 */
extern const char* kdk_cpu_get_freq_MHz();

/**
 * @brief 获取CPU核心数量
 * 
 * @return unsigned int 所有可用的CPU核心数量
 */
extern unsigned int kdk_cpu_get_corenums();

/**
 * @brief 获取CPU对虚拟化的支持
 * 
 * @return const char* 若CPU支持虚拟化，则返回虚拟化技术，如“vmx”；若不支持，返回NULL；返回的是const char*，不要free
 */
extern const char* kdk_cpu_get_virt();

/**
 * @brief 获取CPU线程数
 * 
 * @return unsigned int 返回cpu支持的线程数
 */
extern unsigned int kdk_cpu_get_process();

#ifdef __cplusplus
}
#endif

#endif

/**
 * \example kysdk-system/src/hardware/test/kycpu-test.c
 * 
 */

/**
  * @}
  */
