/*
 * libkysdk-system's Library
 *
 * Copyright (C) 2022, Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yunhe Liu <liuyunhe@kylinos.cn>
 *
 */

#ifndef LIBKYUSBINFO_H
#define LIBKYUSBINFO_H

/**
 * @file libkyusb.h
 * @author liuyunhe (liuyunhe@kylinos.cn)
 * @brief usb信息
 * @version 0.1
 * @date 2023-2-17
 * 
 * @copyright Copyright (c) 2021
 * @defgroup libkysdk-hardware
 * @{
 * 
 */

#ifdef __cplusplus
extern "C"
{
#endif

/*
    * usb设备信息
    */
typedef struct driver_info
{
    char name[32]; //名称
    char type[2 + 1]; //类型
    char pid[4 + 1];
    char vid[4 + 1];
    char serialNo[32]; //序列号
    char devNode[32]; //路径
} DriverInfo, *pDriverInfo;

/*
    * usb设备信息链表
    */
typedef struct driver_list
{
    struct driver_info *data;
    struct driver_list *next;
} DriverInfoList, *pDriverInfoList;

/**
 * @brief 获取外接usb设备信息
 * 
 * @return pDriverInfoList usb设备信息结构体
 */
extern pDriverInfoList kdk_usb_get_list();

/**
 * @brief 释放kdk_usb_get_list返回的usb信息结构体
 * 
 * @param list kdk_usb_get_list的返回值
 */
extern void kdk_usb_free(pDriverInfoList list);

#ifdef __cplusplus
}
#endif

#endif
